/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   parse_line.c                                       :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: gbrochar <gbrochar@student.42.fr>          +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2020/12/22 18:41:28 by gbrochar          #+#    #+#             */
/*   Updated: 2020/12/22 20:15:50 by gbrochar         ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "scop.h"

int					parse_append_data_d(t_buf_d *buffer, char *token)
{
	if (buffer->ptr == buffer->len)
	{
		buffer->data = (double *)realloc(
			buffer->data, sizeof(double) * (buffer->len + BUFFER_SIZE));
		buffer->len += BUFFER_SIZE;
	}
	buffer->data[buffer->ptr] = atof(token);
	buffer->ptr++;
	return (SUCCESS);
}

int					parse_append_data_i(t_buf_i *buffer, char *token)
{
	if (buffer->ptr == buffer->len)
	{
		buffer->data = (int *)realloc(
			buffer->data, sizeof(int) * (buffer->len + BUFFER_SIZE));
		buffer->len += BUFFER_SIZE;
	}
	buffer->data[buffer->ptr] = atoi(token);
	buffer->ptr++;
	return (SUCCESS);
}

t_gl_buf_type		parse_gl_buf_type(char *token)
{
	if (strstr(token, "v"))
		return (VERTEX);
	if (strstr(token, "vt"))
		return (UV);
	if (strstr(token, "vn"))
		return (NORMAL);
	if (strstr(token, "f"))
		return (INDEX);
	return (OTHER);
}

int					parse_token(
	t_env *e,
	char *token,
	t_gl_buf_type gl_buf_type)
{
	int				ret;

	if (gl_buf_type == OTHER)
		ret = BREAK;
	if (gl_buf_type == VERTEX)
		ret = parse_append_data_d(&(e->object.vertices), token);
	if (gl_buf_type == UV)
		ret = parse_append_data_d(&(e->object.uvs), token);
	if (gl_buf_type == NORMAL)
		ret = parse_append_data_d(&(e->object.normals), token);
	if (gl_buf_type == INDEX)
		ret = parse_append_data_i(&(e->object.indices), token);
	return (ret);
}

int					parse_line(t_env *e, char *line)
{
	char			*token;
	t_gl_buf_type	gl_buf_type;
	int				token_count;
	int				ret;

	token_count = 0;
	ret = SUCCESS;
	while ((token = strtok_r(line, " ", &line)))
	{
		if (token_count == 0)
			gl_buf_type = parse_gl_buf_type(token);
		else
			ret = parse_token(e, token, gl_buf_type);
		if (ret == FAILURE)
			return (FAILURE);
		token_count++;
	}
	return (SUCCESS);
}
